/*! \file enc28j60.c \brief Microchip ENC28J60 Ethernet Interface Driver. */
//*****************************************************************************
//
// File Name	: 'enc28j60.c'
// Title		: Microchip ENC28J60 Ethernet Interface Driver
// Author		: Pascal Stang (c)2005
// Created		: 9/22/2005
// Revised		: 9/22/2005
// Version		: 0.1
// Target MCU	: Atmel AVR series
// Editor Tabs	: 4
//
// Description	: This driver provides initialization and transmit/receive
//	functions for the Microchip ENC28J60 10Mb Ethernet Controller and PHY.
// This chip is novel in that it is a full MAC+PHY interface all in a 28-pin
// chip, using an SPI interface to the host processor.
//
//*****************************************************************************

#include "../avrlibdefs.h"
#include "../avrlibtypes.h"
#include "../UART/UART.h"
#include <util/delay.h>

//#include "timer.h"
//#include "rprintf.h"

#include "enc28j60.h"
#include <avr/io.h>
#include "../Interface/interface.h"


u08 Enc28j60Bank;
u16 NextPacketPtr;
#define ENC28J60_CONTROL_PORT	PORTB
#define ENC28J60_CONTROL_DDR	DDRB
#define ENC28J60_CONTROL_CS		0

#define F_CPU        	10000000               		/* Processor Speed */
#define CYCLES_PER_US 	((F_CPU+500000)/1000000) 	/* cpu cycles per microsecond */

void delay(unsigned short us) 
{
    unsigned short  delay_loops;
    register unsigned short  i;

    delay_loops = (us+3)/5*CYCLES_PER_US; // +3 for rounding up (dirty) 

	// one loop takes 5 cpu cycles 
    for (i=0; i < delay_loops; i++) {};
} 

void nicSetMacAddress(u08* macaddr)
{
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, *macaddr++);
	enc28j60Write(MAADR4, *macaddr++);
	enc28j60Write(MAADR3, *macaddr++);
	enc28j60Write(MAADR2, *macaddr++);
	enc28j60Write(MAADR1, *macaddr++);
	enc28j60Write(MAADR0, *macaddr++);
}





u08 enc28j60ReadOp(u08 op, u08 address)
{
	u08 data;
	u08 rxData = 0;
   
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue read command

	write_to_spi(dvc_none, op | (address & ADDR_MASK));

	//SPDR = op | (address & ADDR_MASK);
	//while(!(SPSR & (1<<SPIF)));
	// read data
	//SPDR = 0x00;
	//while(!(SPSR & (1<<SPIF)));

	rxData = transfer_to_spi(dvc_none,0x00);

	// do dummy read if needed
	if(address & 0x80)
	{
		rxData = transfer_to_spi(dvc_none,0x00);
		//SPDR = 0x00;
		//while(!(inb(SPSR) & (1<<SPIF)));
	}
	data = rxData;
	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);

	return data;
}

void enc28j60WriteOp(u08 op, u08 address, u08 data)
{
	// assert CS

	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);

	// issue write command

	//SPDR = op | (address & ADDR_MASK);

	write_to_spi(dvc_none,op | (address & ADDR_MASK));

	//while(!(SPSR & (1<<SPIF)));
	// write data

	//SPDR = data;

	write_to_spi(dvc_none,data);
	//while(!(SPSR & (1<<SPIF)));




	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60ReadBuffer(u16 len, u08* data)
{
	u08 rxData = 0;
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue read command

	write_to_spi(dvc_none,ENC28J60_READ_BUF_MEM);

	//SPDR = ENC28J60_READ_BUF_MEM;
	//while(!(SPSR & (1<<SPIF)));
	while(len-- )
	{
		// read data
		rxData = transfer_to_spi(dvc_none,0x00);

		//SPDR = 0x00;
		//while(!(SPSR & (1<<SPIF)));
		*data++ = rxData;
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60WriteBuffer(u16 len, u08* data)
{
	u08 rxData = 0;
	// assert CS
	ENC28J60_CONTROL_PORT &= ~(1<<ENC28J60_CONTROL_CS);
	
	// issue write command

	write_to_spi(dvc_none,ENC28J60_WRITE_BUF_MEM);
	//SPDR = ENC28J60_WRITE_BUF_MEM;
	//while(!(SPSR & (1<<SPIF)));
	while(len--)
	{
		// write data

		write_to_spi(dvc_none,*data++);
		//SPDR = *data++;
		//while(!(SPSR & (1<<SPIF)));
	}	
	// release CS
	ENC28J60_CONTROL_PORT |= (1<<ENC28J60_CONTROL_CS);
}

void enc28j60SetBank(u08 address)
{
	// set the bank (if needed)
	if((address & BANK_MASK) != Enc28j60Bank)
	{
		// set the bank
		enc28j60WriteOp(ENC28J60_BIT_FIELD_CLR, ECON1, (ECON1_BSEL1|ECON1_BSEL0));
		enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, (address & BANK_MASK)>>5);
		Enc28j60Bank = (address & BANK_MASK);
	}
}

u08 enc28j60Read(u08 address)
{
	// set the bank
	enc28j60SetBank(address);
	// do the read
	return enc28j60ReadOp(ENC28J60_READ_CTRL_REG, address);
}

void enc28j60Write(u08 address, u08 data)
{
	// set the bank
	enc28j60SetBank(address);
	// do the write
	enc28j60WriteOp(ENC28J60_WRITE_CTRL_REG, address, data);
}



void enc28j60PhyWrite(u08 address, u16 data)
{
	// set the PHY register address
	enc28j60Write(MIREGADR, address);
	
	// write the PHY data
	enc28j60Write(MIWRL, data);	
	enc28j60Write(MIWRH, data>>8);

	// wait until the PHY write completes
	while(enc28j60Read(MISTAT) & MISTAT_BUSY);
}

void enc28j60Init(void)
{
	// initialize I/O

	ENC28J60_CONTROL_DDR |= 1<<ENC28J60_CONTROL_CS;
	ENC28J60_CONTROL_PORT |= 1<<ENC28J60_CONTROL_CS;
	

	
	//PORTB |= 1<<PB7; //SCK HI

	//DDRB  |= 1<<PB5 |1<<PB7 | 1<<PB4; // mosi, sck, ss output  ///<--- wird bereits im system.c gemacht
	//DDRB  &= ~(1<<PB6); // miso input
	

	// initialize SPI interface
	// master mode und /2x takt (klappt auch wenn der mega8 mit 16mhz luft !!)
	//SPSR |= 1<<SPR1;

//	uart_puts("- enc28j60init - aktiviere das SPI...\n");
	//SPCR |= 1<<MSTR | 1<<SPE;
	


	// perform system reset
	//uart_puts("- enc28j60init - mache einen Sys. Reset...\n");
	enc28j60WriteOp(ENC28J60_SOFT_RESET, 0, ENC28J60_SOFT_RESET);
	// check CLKRDY bit to see if reset is complete
	//uart_puts("- enc28j60init - warte 50.\n");
	delay(50);
	//uart_puts("- enc28j60init - warte bis ein CLKRDY kommt...\n");
	while(!(enc28j60Read(ESTAT) & ESTAT_CLKRDY));
	//uart_puts("- enc28j60init - CLKRDY ist gekommen.\n");

	// do bank 0 stuff
	// initialize receive buffer
	// 16-bit transfers, must write low byte first
	// set receive buffer start address
	//uart_puts("- enc28j60init - setze RX buffer Addresse.\n");
	NextPacketPtr = RXSTART_INIT;
	//uart_puts("- enc28j60init - Schreibe erstes nibble.\n");
	enc28j60Write(ERXSTL, RXSTART_INIT&0xFF);
	//uart_puts("- enc28j60init - Schreibe zweites nibble.\n");
	enc28j60Write(ERXSTH, RXSTART_INIT>>8);
	// set receive pointer address
	//uart_puts("- enc28j60init - Schreibe erstes nibble - 01.\n");
	enc28j60Write(ERXRDPTL, RXSTART_INIT&0xFF);
	//uart_puts("- enc28j60init - Schreibe zweites nibble - 01\n");
	enc28j60Write(ERXRDPTH, RXSTART_INIT>>8);
	// set receive buffer end
	// ERXND defaults to 0x1FFF (end of ram)
	//uart_puts("- enc28j60init - Schreibe erstes nibble - 02\n");
	enc28j60Write(ERXNDL, RXSTOP_INIT&0xFF);
	//uart_puts("- enc28j60init - Schreibe zweites nibble - 02\n");
	enc28j60Write(ERXNDH, RXSTOP_INIT>>8);
	// set transmit buffer start
	// ETXST defaults to 0x0000 (beginnging of ram)
	//uart_puts("- enc28j60init - Schreibe erstes nibble - 03\n");
	enc28j60Write(ETXSTL, TXSTART_INIT&0xFF);
	//uart_puts("- enc28j60init - Schreibe zweites nibble - 03\n");
	enc28j60Write(ETXSTH, TXSTART_INIT>>8);

	// do bank 2 stuff
	// enable MAC receive
	enc28j60Write(MACON1, MACON1_MARXEN|MACON1_TXPAUS|MACON1_RXPAUS);
	// bring MAC out of reset
	enc28j60Write(MACON2, 0x00);
	// enable automatic padding and CRC operations
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, MACON3, MACON3_PADCFG0|MACON3_TXCRCEN|MACON3_FRMLNEN);

	// set inter-frame gap (non-back-to-back)
	enc28j60Write(MAIPGL, 0x12);
	enc28j60Write(MAIPGH, 0x0C);
	// set inter-frame gap (back-to-back)
	enc28j60Write(MABBIPG, 0x12);
	// Set the maximum packet size which the controller will accept
	enc28j60Write(MAMXFLL, MAX_FRAMELEN&0xFF);	
	enc28j60Write(MAMXFLH, MAX_FRAMELEN>>8);

	// do bank 3 stuff
	// write MAC address
	// NOTE: MAC address in ENC28J60 is byte-backward
	enc28j60Write(MAADR5, ENC28J60_MAC0);
	enc28j60Write(MAADR4, ENC28J60_MAC1);
	enc28j60Write(MAADR3, ENC28J60_MAC2);
	enc28j60Write(MAADR2, ENC28J60_MAC3);
	enc28j60Write(MAADR1, ENC28J60_MAC4);
	enc28j60Write(MAADR0, ENC28J60_MAC5);

	// no loopback of transmitted frames
	enc28j60PhyWrite(PHCON2, PHCON2_HDLDIS);

	// switch to bank 0
	enc28j60SetBank(ECON1);
	// enable interrutps
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, EIE, EIE_INTIE|EIE_PKTIE);
	// enable packet reception
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_RXEN);

}

void enc28j60PacketSend(unsigned int len, unsigned char* packet)
{
	// Set the write pointer to start of transmit buffer area
	enc28j60Write(EWRPTL, TXSTART_INIT);
	enc28j60Write(EWRPTH, TXSTART_INIT>>8);
	// Set the TXND pointer to correspond to the packet size given
	enc28j60Write(ETXNDL, (TXSTART_INIT+len));
	enc28j60Write(ETXNDH, (TXSTART_INIT+len)>>8);

	// write per-packet control byte
	enc28j60WriteOp(ENC28J60_WRITE_BUF_MEM, 0, 0x00);

	// copy the packet into the transmit buffer
	enc28j60WriteBuffer(len, packet);
	
	// send the contents of the transmit buffer onto the network
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON1, ECON1_TXRTS);
}

unsigned int enc28j60PacketReceive(unsigned int maxlen, unsigned char* packet)
{
	u16 rxstat;
	u16 len;

	// check if a packet has been received and buffered
	if( !(enc28j60Read(EIR) & EIR_PKTIF) )
		return 0;
	
	// Make absolutely certain that any previous packet was discarded	
	//if( WasDiscarded == FALSE)
	//	MACDiscardRx();

	// Set the read pointer to the start of the received packet
	enc28j60Write(ERDPTL, (NextPacketPtr));
	enc28j60Write(ERDPTH, (NextPacketPtr)>>8);
	// read the next packet pointer
	NextPacketPtr  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	NextPacketPtr |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the packet length
	len  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	len |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;
	// read the receive status
	rxstat  = enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0);
	rxstat |= enc28j60ReadOp(ENC28J60_READ_BUF_MEM, 0)<<8;

	// limit retrieve length
	// (we reduce the MAC-reported length by 4 to remove the CRC)
	len = MIN(len, maxlen);

	// copy the packet from the receive buffer
	enc28j60ReadBuffer(len, packet);

	// Move the RX read pointer to the start of the next received packet
	// This frees the memory we just read out
	enc28j60Write(ERXRDPTL, (NextPacketPtr));
	enc28j60Write(ERXRDPTH, (NextPacketPtr)>>8);

	// decrement the packet counter indicate we are done with this packet
	enc28j60WriteOp(ENC28J60_BIT_FIELD_SET, ECON2, ECON2_PKTDEC);

	return len;
}




